@@ -26,9 +26,9 @@
 
 FPLLL_BEGIN_NAMESPACE
 
-template <class FT>
-BKZReduction<FT>::BKZReduction(MatGSO<Integer, FT> &m, LLLReduction<Integer, FT> &lll_obj,
-                               const BKZParam &param)
+template <class ZT, class FT>
+BKZReduction<ZT, FT>::BKZReduction(MatGSO<ZT, FT> &m, LLLReduction<ZT, FT> &lll_obj,
+                                   const BKZParam &param)
     : status(RED_SUCCESS), nodes(0), param(param), m(m), lll_obj(lll_obj), algorithm(NULL),
       cputime_start(0)
 {
@@ -38,9 +38,10 @@ BKZReduction<FT>::BKZReduction(MatGSO<Integer, FT> &m, LLLReduction<Integer, FT>
   this->delta = param.delta;
 }
 
-template <class FT> BKZReduction<FT>::~BKZReduction() {}
+template <class ZT, class FT> BKZReduction<ZT, FT>::~BKZReduction() {}
 
-template <class FT> void BKZReduction<FT>::rerandomize_block(int min_row, int max_row, int density)
+template <class ZT, class FT>
+void BKZReduction<ZT, FT>::rerandomize_block(int min_row, int max_row, int density)
 {
   if (max_row - min_row < 2)
     return;
@@ -78,8 +79,9 @@ template <class FT> void BKZReduction<FT>::rerandomize_block(int min_row, int ma
   return;
 }
 
-template <class FT>
-const Pruning &BKZReduction<FT>::get_pruning(int kappa, int block_size, const BKZParam &par) const
+template <class ZT, class FT>
+const PruningParams &BKZReduction<ZT, FT>::get_pruning(int kappa, int block_size,
+                                                       const BKZParam &par) const
 {
 
   FPLLL_DEBUG_CHECK(param.strategies.size() > block_size);
@@ -95,8 +97,8 @@ const Pruning &BKZReduction<FT>::get_pruning(int kappa, int block_size, const BK
                            gh_max_dist.get_d() * pow(2, max_dist_expo));
 }
 
-template <class FT>
-bool BKZReduction<FT>::svp_preprocessing(int kappa, int block_size, const BKZParam &param)
+template <class ZT, class FT>
+bool BKZReduction<ZT, FT>::svp_preprocessing(int kappa, int block_size, const BKZParam &param)
 {
   bool clean = true;
 
@@ -110,6 +112,7 @@ bool BKZReduction<FT>::svp_preprocessing(int kappa, int block_size, const BKZPar
   if (lll_obj.n_swaps > 0)
     clean = false;
 
+  // run one tour of recursive preprocessing
   auto &preproc = param.strategies[block_size].preprocessing_block_sizes;
   for (auto it = preproc.begin(); it != preproc.end(); ++it)
   {
@@ -121,8 +124,9 @@ bool BKZReduction<FT>::svp_preprocessing(int kappa, int block_size, const BKZPar
   return clean;
 }
 
-template <class FT>
-bool BKZReduction<FT>::svp_postprocessing(int kappa, int block_size, const vector<FT> &solution)
+template <class ZT, class FT>
+bool BKZReduction<ZT, FT>::svp_postprocessing(int kappa, int block_size, const vector<FT> &solution,
+                                              bool dual)
 {
   // Is it already in the basis ?
   int nz_vectors = 0, i_vector = -1;
@@ -137,77 +141,86 @@ bool BKZReduction<FT>::svp_postprocessing(int kappa, int block_size, const vecto
   }
   // nz_vectors is the number of nonzero coordinates
   // i_vector is the largest index for a \pm 1 coordinate
-
   FPLLL_DEBUG_CHECK(nz_vectors > 0);
 
+  int pos = dual ? kappa + block_size - 1 : kappa;
   if (nz_vectors == 1)
   {
     // Yes, it is another vector
-    FPLLL_DEBUG_CHECK(i_vector != -1 && i_vector != 0);
-    m.move_row(kappa + i_vector, kappa);
-    if (!lll_obj.lll(0, kappa, kappa + 1, 0))
-      throw lll_obj.status;
+    FPLLL_DEBUG_CHECK(i_vector != -1 && i_vector != (pos - kappa));
+    m.move_row(kappa + i_vector, pos);
   }
   else if (i_vector != -1)
   {
-    // No, but one coordinate is equal to \pm 1, making
-    // linear dependency easy to fix too.
-    int d = m.d;
-    m.create_row();
-    m.row_op_begin(d, d + 1);
-    for (int i = 0; i < block_size; i++)
+    // No, but one coordinate is equal to \pm 1, we'll
+    // just compute the new vector in that position.
+    int sol_i = solution[i_vector].get_si();
+    if (dual)
     {
-      m.row_addmul(d, kappa + i, solution[i]);
+      sol_i *= -1;
+      m.row_op_begin(kappa, kappa + block_size);
     }
-    m.row_op_end(d, d + 1);
-    m.move_row(d, kappa);
-    m.move_row(kappa + i_vector + 1, d);
-    m.remove_last_row();
-    if (!lll_obj.lll(0, kappa, kappa + 1, 0))
-      throw lll_obj.status;
+    else
+    {
+      // in case of primal reduction, we can restrict invalidation to
+      // the one vector we're adding rows to
+      m.row_op_begin(kappa + i_vector, kappa + i_vector + 1);
+    }
+
+    for (int i = 0; i < block_size; ++i)
+    {
+      if (!solution[i].is_zero() && (i != i_vector))
+      {
+        if (dual)
+        {
+          m.row_addmul(kappa + i, kappa + i_vector, sol_i * solution[i]);
+        }
+        else
+        {
+          m.row_addmul(kappa + i_vector, kappa + i, sol_i * solution[i]);
+        }
+      }
+    }
+
+    if (dual)
+    {
+      m.row_op_end(kappa, kappa + block_size);
+    }
+    else
+    {
+      m.row_op_end(kappa + i_vector, kappa + i_vector + 1);
+    }
+
+    m.move_row(kappa + i_vector, pos);
   }
   else
   {
     // No, general case
-    int d = m.d;
-    m.create_row();
-    m.row_op_begin(d, d + 1);
-    for (int i = 0; i < block_size; i++)
-    {
-      m.row_addmul(d, kappa + i, solution[i]);
-    }
-    m.row_op_end(d, d + 1);
-    m.move_row(d, kappa);
-    if (!lll_obj.lll(0, kappa, kappa + block_size + 1, 0))
-      throw lll_obj.status;
-    FPLLL_DEBUG_CHECK(m.b[kappa + block_size].is_zero());
-    m.move_row(kappa + block_size, d);
-    m.remove_last_row();
+    svp_postprocessing_generic(kappa, block_size, solution, dual);
   }
   return false;
 }
 
-template <class FT>
-bool BKZReduction<FT>::dsvp_postprocessing(int kappa, int block_size, const vector<FT> &solution)
+template <class ZT, class FT>
+bool BKZReduction<ZT, FT>::svp_postprocessing_generic(int kappa, int block_size,
+                                                      const vector<FT> &solution, bool dual)
 {
   vector<FT> x = solution;
-
-  int d = block_size;
-  m.row_op_begin(kappa, kappa + d);
+  int d        = block_size;
   // don't want to deal with negativ coefficients
   for (int i = 0; i < d; i++)
   {
     if (x[i] < 0)
     {
       x[i].neg(x[i]);
-      for (int j = 0; j < m.b.get_cols(); j++)
-      {
-        m.b[i + kappa][j].neg(m.b[i + kappa][j]);
-      }
+      m.negate_row_of_b(i + kappa);
     }
   }
 
+  m.row_op_begin(kappa, kappa + d);
   // tree based gcd computation on x, performing operations also on b
+  // (or the dual operations in case of primal [sounds weird,
+  // but is correct] svp reduction)
   int off = 1;
   int k;
   while (off < d)
@@ -220,7 +233,7 @@ bool BKZReduction<FT>::dsvp_postprocessing(int kappa, int block_size, const vect
         if (x[k] < x[k - off])
         {
           x[k].swap(x[k - off]);
-          m.b.swap_rows(kappa + k, kappa + k - off);
+          m.row_swap(kappa + k - off, kappa + k);
         }
 
         while (!x[k - off].is_zero())
@@ -228,30 +241,38 @@ bool BKZReduction<FT>::dsvp_postprocessing(int kappa, int block_size, const vect
           while (x[k - off] <= x[k])
           {
             x[k] = x[k] - x[k - off];
-            m.b[kappa + k].sub(m.b[kappa + k - off]);
+            if (dual)
+            {
+              m.row_sub(kappa + k, kappa + k - off);
+            }
+            else
+            {
+              m.row_add(kappa + k - off, kappa + k);
+            }
           }
 
           x[k].swap(x[k - off]);
-          m.b.swap_rows(kappa + k, kappa + k - off);
+          m.row_swap(kappa + k - off, kappa + k);
         }
       }
       k -= 2 * off;
     }
     off *= 2;
   }
-
   m.row_op_end(kappa, kappa + d);
-  if (!lll_obj.lll(kappa, kappa, kappa + d, 0))
+
+  // the gcd computation will leave the desired vector in last
+  // position, so in case of primal reduction we need to move it up
+  if (!dual)
   {
-    return set_status(lll_obj.status);
+    m.move_row(kappa + d - 1, kappa);
   }
   return false;
 }
 
-template <class FT>
-bool BKZReduction<FT>::svp_reduction(int kappa, int block_size, const BKZParam &par, bool dual)
+template <class ZT, class FT>
+bool BKZReduction<ZT, FT>::svp_reduction(int kappa, int block_size, const BKZParam &par, bool dual)
 {
-
   int first = dual ? kappa + block_size - 1 : kappa;
 
   // ensure we are computing something sensible.
@@ -261,6 +282,9 @@ bool BKZReduction<FT>::svp_reduction(int kappa, int block_size, const BKZParam &
   // already in the basis). if size reduction is not called,
   // old_first might be incorrect (e.g. close to 0) and the function
   // will return an incorrect clean flag
+  // WARNING: do not try to increase the size reduction beyond first.
+  // GSO might be invalid beyond this and this can cause numerical issues
+  // and even nullpointers!
   if (!lll_obj.size_reduction(0, first + 1, 0))
   {
     throw std::runtime_error(RED_STATUS_STR[lll_obj.status]);
@@ -281,6 +305,7 @@ bool BKZReduction<FT>::svp_reduction(int kappa, int block_size, const BKZParam &
 
     svp_preprocessing(kappa, block_size, par);
 
+    // compute enumeration radius
     long max_dist_expo;
     FT max_dist = m.get_r_exp(first, first, max_dist_expo);
     if (dual)
@@ -296,23 +321,18 @@ bool BKZReduction<FT>::svp_reduction(int kappa, int block_size, const BKZParam &
       adjust_radius_to_gh_bound(max_dist, max_dist_expo, block_size, root_det, par.gh_factor);
     }
 
-    const Pruning &pruning = get_pruning(kappa, block_size, par);
+    const PruningParams &pruning = get_pruning(kappa, block_size, par);
 
     FPLLL_DEBUG_CHECK(pruning.metric == PRUNER_METRIC_PROBABILITY_OF_SHORTEST)
-
     evaluator.solutions.clear();
-    Enumeration<FT> enum_obj(m, evaluator);
+    Enumeration<ZT, FT> enum_obj(m, evaluator);
     enum_obj.enumerate(kappa, kappa + block_size, max_dist, max_dist_expo, vector<FT>(),
                        vector<enumxt>(), pruning.coefficients, dual);
     nodes += enum_obj.get_nodes();
 
     if (!evaluator.empty())
     {
-      if (dual)
-        dsvp_postprocessing(kappa, block_size, evaluator.begin()->second);
-      else
-        svp_postprocessing(kappa, block_size, evaluator.begin()->second);
-
+      svp_postprocessing(kappa, block_size, evaluator.begin()->second, dual);
       rerandomize = false;
     }
     else
@@ -326,15 +346,19 @@ bool BKZReduction<FT>::svp_reduction(int kappa, int block_size, const BKZParam &
   {
     throw std::runtime_error(RED_STATUS_STR[lll_obj.status]);
   }
+
+  // in order to check if we made progress, we compare the new shortest vector to the
+  // old one (note that simply checking clean flags is not sufficient since
+  // preprocessing can have changed things but we don't know if it made progress)
   long new_first_expo;
   FT new_first = m.get_r_exp(first, first, new_first_expo);
   new_first.mul_2si(new_first, new_first_expo - old_first_expo);
   return (dual) ? (old_first >= new_first) : (old_first <= new_first);
 }
 
-template <class FT>
-bool BKZReduction<FT>::tour(const int loop, int &kappa_max, const BKZParam &par, int min_row,
-                            int max_row)
+template <class ZT, class FT>
+bool BKZReduction<ZT, FT>::tour(const int loop, int &kappa_max, const BKZParam &par, int min_row,
+                                int max_row)
 {
   bool clean = true;
   clean &= trunc_tour(kappa_max, par, min_row, max_row);
@@ -347,18 +371,15 @@ bool BKZReduction<FT>::tour(const int loop, int &kappa_max, const BKZParam &par,
 
   if (par.flags & BKZ_DUMP_GSO)
   {
-    std::ostringstream prefix;
-    prefix << "End of BKZ loop " << std::setw(4) << loop;
-    prefix << " (" << std::fixed << std::setw(9) << std::setprecision(3)
-           << (cputime() - cputime_start) * 0.001 << "s)";
-    dump_gso(par.dump_gso_filename, prefix.str());
+    dump_gso(par.dump_gso_filename, true, "End of BKZ loop", loop,
+             (cputime() - cputime_start) * 0.001);
   }
 
   return clean;
 }
 
-template <class FT>
-bool BKZReduction<FT>::trunc_tour(int &kappa_max, const BKZParam &par, int min_row, int max_row)
+template <class ZT, class FT>
+bool BKZReduction<ZT, FT>::trunc_tour(int &kappa_max, const BKZParam &par, int min_row, int max_row)
 {
   bool clean     = true;
   int block_size = par.block_size;
@@ -376,8 +397,8 @@ bool BKZReduction<FT>::trunc_tour(int &kappa_max, const BKZParam &par, int min_r
   return clean;
 }
 
-template <class FT>
-bool BKZReduction<FT>::trunc_dtour(const BKZParam &par, int min_row, int max_row)
+template <class ZT, class FT>
+bool BKZReduction<ZT, FT>::trunc_dtour(const BKZParam &par, int min_row, int max_row)
 {
   bool clean     = true;
   int block_size = par.block_size;
@@ -390,8 +411,8 @@ bool BKZReduction<FT>::trunc_dtour(const BKZParam &par, int min_row, int max_row
   return clean;
 }
 
-template <class FT>
-bool BKZReduction<FT>::hkz(int &kappa_max, const BKZParam &param, int min_row, int max_row)
+template <class ZT, class FT>
+bool BKZReduction<ZT, FT>::hkz(int &kappa_max, const BKZParam &param, int min_row, int max_row)
 {
   bool clean = true;
   for (int kappa = min_row; kappa < max_row - 1; ++kappa)
@@ -406,11 +427,16 @@ bool BKZReduction<FT>::hkz(int &kappa_max, const BKZParam &param, int min_row, i
     }
   }
 
+  // this line fixes fpylll issue 73 with stalling BKZ instances
+  // test basis: tests/lattices/stalling_93_53.txt. Test with command
+  // fplll -a bkz -b 53 -s strategies/default.json -f double tests/lattices/stalling_93_53.txt
+  lll_obj.size_reduction(max_row - 1, max_row, max_row - 2);
+
   return clean;
 }
 
-template <class FT>
-bool BKZReduction<FT>::sd_tour(const int loop, const BKZParam &par, int min_row, int max_row)
+template <class ZT, class FT>
+bool BKZReduction<ZT, FT>::sd_tour(const int loop, const BKZParam &par, int min_row, int max_row)
 {
   int dummy_kappa_max = num_rows;
   bool clean          = true;
@@ -424,18 +450,15 @@ bool BKZReduction<FT>::sd_tour(const int loop, const BKZParam &par, int min_row,
 
   if (par.flags & BKZ_DUMP_GSO)
   {
-    std::ostringstream prefix;
-    prefix << "End of SD-BKZ loop " << std::setw(4) << loop;
-    prefix << " (" << std::fixed << std::setw(9) << std::setprecision(3)
-           << (cputime() - cputime_start) * 0.001 << "s)";
-    dump_gso(par.dump_gso_filename, prefix.str());
+    dump_gso(par.dump_gso_filename, true, "End of SD-BKZ loop", loop,
+             (cputime() - cputime_start) * 0.001);
   }
 
   return clean;
 }
 
-template <class FT>
-bool BKZReduction<FT>::slide_tour(const int loop, const BKZParam &par, int min_row, int max_row)
+template <class ZT, class FT>
+bool BKZReduction<ZT, FT>::slide_tour(const int loop, const BKZParam &par, int min_row, int max_row)
 {
   int p = (max_row - min_row) / par.block_size;
   if ((max_row - min_row) % par.block_size)
@@ -444,13 +467,24 @@ bool BKZReduction<FT>::slide_tour(const int loop, const BKZParam &par, int min_r
   do
   {
     clean = true;
-    // SVP reduction takes care of the LLL reduction as long as BKZ_BOUNDED_LLL is off
     for (int i = 0; i < p; ++i)
     {
       int kappa      = min_row + i * par.block_size;
       int block_size = min(max_row - kappa, par.block_size);
       clean &= svp_reduction(kappa, block_size, par);
     }
+    // SVP reduction takes care of the LLL reduction if BKZ_BOUNDED_LLL is off
+    if (par.flags & BKZ_BOUNDED_LLL)
+    {
+      if (!lll_obj.lll(min_row, min_row, max_row, 0))
+      {
+        throw std::runtime_error(RED_STATUS_STR[lll_obj.status]);
+      }
+      if (lll_obj.n_swaps > 0)
+      {
+        clean = false;
+      }
+    }
   } while (!clean);
 
   for (int i = 0; i < p - 1; ++i)
@@ -468,13 +502,11 @@ bool BKZReduction<FT>::slide_tour(const int loop, const BKZParam &par, int min_r
 
   if (par.flags & BKZ_DUMP_GSO)
   {
-    std::ostringstream prefix;
-    prefix << "End of SLD loop " << std::setw(4) << loop;
-    prefix << " (" << std::fixed << std::setw(9) << std::setprecision(3)
-           << (cputime() - cputime_start) * 0.001 << "s)";
-    dump_gso(par.dump_gso_filename, prefix.str());
+    dump_gso(par.dump_gso_filename, true, "End of SLD loop", loop,
+             (cputime() - cputime_start) * 0.001);
   }
 
+  // we check the potential function to see if we made progress
   if (new_potential >= sld_potential)
     return true;
 
@@ -482,7 +514,7 @@ bool BKZReduction<FT>::slide_tour(const int loop, const BKZParam &par, int min_r
   return false;
 }
 
-template <class FT> bool BKZReduction<FT>::bkz()
+template <class ZT, class FT> bool BKZReduction<ZT, FT>::bkz()
 {
   int flags        = param.flags;
   int final_status = RED_SUCCESS;
@@ -498,9 +530,7 @@ template <class FT> bool BKZReduction<FT>::bkz()
 
   if (flags & BKZ_DUMP_GSO)
   {
-    std::ostringstream prefix;
-    prefix << "Input";
-    dump_gso(param.dump_gso_filename, prefix.str(), false);
+    dump_gso(param.dump_gso_filename, false, "Input", -1, 0.0);
   }
 
   if (param.block_size < 2)
@@ -508,7 +538,7 @@ template <class FT> bool BKZReduction<FT>::bkz()
 
   int i = 0;
 
-  BKZAutoAbort<FT> auto_abort(m, num_rows);
+  BKZAutoAbort<ZT, FT> auto_abort(m, num_rows);
 
   if (sd && !(flags & (BKZ_MAX_LOOPS | BKZ_MAX_TIME | BKZ_AUTO_ABORT)))
   {
@@ -580,17 +610,23 @@ template <class FT> bool BKZReduction<FT>::bkz()
       return set_status(e);
     }
 
+    // if we do hkz reduction, we only need one tour
     if (clean || param.block_size >= num_rows)
       break;
   }
 
+  // some post processing
   int dummy_kappa_max = num_rows;
   if (sd)
   {
     try
     {
+      // hkz reduce the last window, which sd leaves unreduced
       hkz(dummy_kappa_max, param, num_rows - param.block_size, num_rows);
-      print_tour(i, 0, num_rows);
+      if (flags & BKZ_DUMP_GSO)
+      {
+        print_tour(i, 0, num_rows);
+      }
     }
     catch (RedStatus &e)
     {
@@ -601,6 +637,7 @@ template <class FT> bool BKZReduction<FT>::bkz()
   {
     try
     {
+      // hkz reduce the blocks (which are otherwise only svp and dual svp reduced)
       int p = num_rows / param.block_size;
       if (num_rows % param.block_size)
         ++p;
@@ -610,7 +647,10 @@ template <class FT> bool BKZReduction<FT>::bkz()
         int end   = min(num_rows, kappa + param.block_size - 1);
         hkz(dummy_kappa_max, param, kappa, end);
       }
-      print_tour(i, 0, num_rows);
+      if (flags & BKZ_DUMP_GSO)
+      {
+        print_tour(i, 0, num_rows);
+      }
     }
     catch (RedStatus &e)
     {
@@ -620,19 +660,16 @@ template <class FT> bool BKZReduction<FT>::bkz()
 
   if (flags & BKZ_DUMP_GSO)
   {
-    std::ostringstream prefix;
-    prefix << "Output ";
-    prefix << " (" << std::fixed << std::setw(9) << std::setprecision(3)
-           << (cputime() - cputime_start) * 0.001 << "s)";
-    dump_gso(param.dump_gso_filename, prefix.str());
+    dump_gso(param.dump_gso_filename, true, "Output", -1, (cputime() - cputime_start) * 0.001);
   }
   return set_status(final_status);
 }
 
-template <class FT> void BKZReduction<FT>::print_tour(const int loop, int min_row, int max_row)
+template <class ZT, class FT>
+void BKZReduction<ZT, FT>::print_tour(const int loop, int min_row, int max_row)
 {
   FT r0;
-  Float fr0;
+  FP_NR<mpfr_t> fr0;
   long expo;
   r0  = m.get_r_exp(min_row, min_row, expo);
   fr0 = r0.get_d();
@@ -645,7 +682,8 @@ template <class FT> void BKZReduction<FT>::print_tour(const int loop, int min_ro
   cerr << ", log2(nodes) = " << std::setw(9) << std::setprecision(6) << log2(nodes) << endl;
 }
 
-template <class FT> void BKZReduction<FT>::print_params(const BKZParam &param, ostream &out)
+template <class ZT, class FT>
+void BKZReduction<ZT, FT>::print_params(const BKZParam &param, ostream &out)
 {
   out << "block size: " << std::setw(3) << param.block_size << ", ";
   out << "flags: 0x" << std::setw(4) << setfill('0') << std::hex << param.flags << ", " << std::dec
@@ -666,7 +704,7 @@ template <class FT> void BKZReduction<FT>::print_params(const BKZParam &param, o
   out << endl;
 }
 
-template <class FT> bool BKZReduction<FT>::set_status(int new_status)
+template <class ZT, class FT> bool BKZReduction<ZT, FT>::set_status(int new_status)
 {
   status = new_status;
   if (param.flags & BKZ_VERBOSE)
@@ -679,29 +717,81 @@ template <class FT> bool BKZReduction<FT>::set_status(int new_status)
   return status == RED_SUCCESS;
 }
 
-template <class FT>
-void BKZReduction<FT>::dump_gso(const std::string &filename, const std::string &prefix, bool append)
+// Generate the json file by hand to generate a flexible human-readable file.
+// TODO: think about use io/json.hpp
+template <class ZT, class FT>
+void BKZReduction<ZT, FT>::dump_gso(const std::string &filename, bool append,
+                                    const std::string &step, const int loop, const double time)
 {
   ofstream dump;
-  if (append)
-    dump.open(filename.c_str(), std::ios_base::app);
-  else
-    dump.open(filename.c_str());
-  dump << std::setw(4) << prefix << ": ";
+  // Enable exceptions
+  dump.exceptions(ios_base::failbit | ios_base::badbit);
+
+  try
+  {
+    if (append)
+    {
+      dump.open(filename.c_str(), std::ios_base::app);
+    }
+    else
+    {
+      dump.open(filename.c_str());
+      dump << "[" << endl;
+    }
+  }
+  catch (const ios_base::failure &e)
+  {
+    cerr << "Cannot open " << filename << endl;
+    throw;
+  }
+
+  try
+  {
+    dump << string(8, ' ') << "{" << endl;
+    dump << string(16, ' ') << "\"step\": \"" << step << "\"," << endl;
+    dump << string(16, ' ') << "\"loop\": " << loop << "," << endl;
+    dump << string(16, ' ') << "\"time\": " << time << "," << endl;
+  }
+  catch (const ios_base::failure &e)
+  {
+    cerr << "Cannot open " << filename << endl;
+    throw;
+  }
+
   FT f, log_f;
   long expo;
+  stringstream ss;
   for (int i = 0; i < num_rows; i++)
   {
     m.update_gso_row(i);
     f = m.get_r_exp(i, i, expo);
     log_f.log(f, GMP_RNDU);
-    dump << std::setprecision(8) << log_f.get_d() + expo * std::log(2.0) << " ";
+    ss << std::setprecision(8) << log_f.get_d() + expo * std::log(2.0) << ", ";
   }
-  dump << std::endl;
+  string s = ss.str();
+  try
+  {
+    dump << string(16, ' ') << "\"norms\": [" << s.substr(0, s.size() - 2) << "]" << endl;
+    dump << string(8, ' ') << "}";
+    if (step.compare("Output") == 0)
+    {
+      dump << endl << "]";
+    }
+    else
+    {
+      dump << "," << endl;
+    }
+  }
+  catch (const ios_base::failure &e)
+  {
+    cerr << "Cannot open " << filename << endl;
+    throw;
+  }
+
   dump.close();
 }
 
-template <class FT> bool BKZAutoAbort<FT>::test_abort(double scale, int maxNoDec)
+template <class ZT, class FT> bool BKZAutoAbort<ZT, FT>::test_abort(double scale, int maxNoDec)
 {
   double new_slope = -m.get_current_slope(start_row, num_rows);
   if (no_dec == -1 || new_slope < scale * old_slope)
@@ -712,35 +802,52 @@ template <class FT> bool BKZAutoAbort<FT>::test_abort(double scale, int maxNoDec
   return no_dec >= maxNoDec;
 }
 
-/**
- * call LLLReduction() and then BKZReduction.
- */
-
+// call LLLReduction() and then BKZReduction.
 template <class FT>
-int bkz_reduction_f(IntMatrix &b, const BKZParam &param, int sel_ft, double lll_delta, IntMatrix &u,
-                    IntMatrix &u_inv)
+int bkz_reduction_f(ZZ_mat<mpz_t> &b, const BKZParam &param, int sel_ft, double lll_delta,
+                    ZZ_mat<mpz_t> &u, ZZ_mat<mpz_t> &u_inv)
 {
   int gso_flags = 0;
   if (b.get_rows() == 0 || b.get_cols() == 0)
     return RED_SUCCESS;
   if (sel_ft == FT_DOUBLE || sel_ft == FT_LONG_DOUBLE)
     gso_flags |= GSO_ROW_EXPO;
-  MatGSO<Integer, FT> m_gso(b, u, u_inv, gso_flags);
-  LLLReduction<Integer, FT> lll_obj(m_gso, lll_delta, LLL_DEF_ETA, LLL_DEFAULT);
-  BKZReduction<FT> bkz_obj(m_gso, lll_obj, param);
-  bkz_obj.bkz();
-  return bkz_obj.status;
+  ZZ_mat<long> bl;
+  // we check if we can convert the basis to long integers for performance
+  if (convert<long, mpz_t>(bl, b, 10))
+  {
+    ZZ_mat<long> ul;
+    convert<long, mpz_t>(ul, u, 0);
+    ZZ_mat<long> ul_inv;
+    convert<long, mpz_t>(ul_inv, u_inv, 0);
+
+    MatGSO<Z_NR<long>, FT> m_gso(bl, ul, ul_inv, gso_flags);
+    LLLReduction<Z_NR<long>, FT> lll_obj(m_gso, lll_delta, LLL_DEF_ETA, LLL_DEFAULT);
+    BKZReduction<Z_NR<long>, FT> bkz_obj(m_gso, lll_obj, param);
+    bkz_obj.bkz();
+
+    convert<mpz_t, long>(b, bl, 0);
+    convert<mpz_t, long>(u, ul, 0);
+    convert<mpz_t, long>(u_inv, ul_inv, 0);
+    return bkz_obj.status;
+  }
+  else
+  {
+    MatGSO<Z_NR<mpz_t>, FT> m_gso(b, u, u_inv, gso_flags);
+    LLLReduction<Z_NR<mpz_t>, FT> lll_obj(m_gso, lll_delta, LLL_DEF_ETA, LLL_DEFAULT);
+    BKZReduction<Z_NR<mpz_t>, FT> bkz_obj(m_gso, lll_obj, param);
+    bkz_obj.bkz();
+    return bkz_obj.status;
+  }
 }
 
-/**
- * interface called from call_bkz() from main.cpp.
- */
-int bkz_reduction(IntMatrix *B, IntMatrix *U, const BKZParam &param, FloatType float_type,
+// interface called from call_bkz() from main.cpp.
+int bkz_reduction(ZZ_mat<mpz_t> *B, ZZ_mat<mpz_t> *U, const BKZParam &param, FloatType float_type,
                   int precision)
 {
-  IntMatrix empty_mat;
-  IntMatrix &u     = U ? *U : empty_mat;
-  IntMatrix &u_inv = empty_mat;
+  ZZ_mat<mpz_t> empty_mat;
+  ZZ_mat<mpz_t> &u     = U ? *U : empty_mat;
+  ZZ_mat<mpz_t> &u_inv = empty_mat;
   FPLLL_CHECK(B, "B == NULL in bkzReduction");
 
   if (U && (!u.empty()))
@@ -800,17 +907,22 @@ int bkz_reduction(IntMatrix *B, IntMatrix *U, const BKZParam &param, FloatType f
   }
   else
   {
-    FPLLL_ABORT("Compiled without support for BKZ reduction with " << FLOAT_TYPE_STR[sel_ft]);
+    if (0 <= sel_ft && sel_ft <= FT_MPFR)
+    {
+      // it's a valid choice but we don't have support for it
+      FPLLL_ABORT("Compiled without support for BKZ reduction with " << FLOAT_TYPE_STR[sel_ft]);
+    }
+    else
+    {
+      // it's an invalid choice
+      FPLLL_ABORT("Floating point type " << sel_ft << "not supported in BKZ");
+    }
   }
   zeros_first(*B, u, u_inv);
   return status;
 }
 
-/**
- * We define BKZ/HKZ for each input type instead of using a template,
- * in order to force the compiler to instantiate the functions.
- */
-int bkz_reduction(IntMatrix &b, int block_size, int flags, FloatType float_type, int precision)
+int bkz_reduction(ZZ_mat<mpz_t> &b, int block_size, int flags, FloatType float_type, int precision)
 {
   vector<Strategy> strategies;
   BKZParam param(block_size, strategies);
@@ -818,8 +930,8 @@ int bkz_reduction(IntMatrix &b, int block_size, int flags, FloatType float_type,
   return bkz_reduction(&b, NULL, param, float_type, precision);
 }
 
-int bkz_reduction(IntMatrix &b, IntMatrix &u, int block_size, int flags, FloatType float_type,
-                  int precision)
+int bkz_reduction(ZZ_mat<mpz_t> &b, ZZ_mat<mpz_t> &u, int block_size, int flags,
+                  FloatType float_type, int precision)
 {
   vector<Strategy> strategies;
   BKZParam param(block_size, strategies);
@@ -827,7 +939,7 @@ int bkz_reduction(IntMatrix &b, IntMatrix &u, int block_size, int flags, FloatTy
   return bkz_reduction(&b, &u, param, float_type, precision);
 }
 
-int hkz_reduction(IntMatrix &b, int flags, FloatType float_type, int precision)
+int hkz_reduction(ZZ_mat<mpz_t> &b, int flags, FloatType float_type, int precision)
 {
   vector<Strategy> strategies;
   BKZParam param(b.get_rows(), strategies);
@@ -840,28 +952,46 @@ int hkz_reduction(IntMatrix &b, int flags, FloatType float_type, int precision)
 
 /** enforce instantiation of complete templates **/
 
-template class BKZReduction<FP_NR<double>>;
-template class BKZAutoAbort<FP_NR<double>>;
+template class BKZReduction<Z_NR<mpz_t>, FP_NR<double>>;
+template class BKZAutoAbort<Z_NR<mpz_t>, FP_NR<double>>;
+
+template class BKZReduction<Z_NR<long>, FP_NR<double>>;
+template class BKZAutoAbort<Z_NR<long>, FP_NR<double>>;
 
 #ifdef FPLLL_WITH_LONG_DOUBLE
-template class BKZReduction<FP_NR<long double>>;
-template class BKZAutoAbort<FP_NR<long double>>;
+template class BKZReduction<Z_NR<mpz_t>, FP_NR<long double>>;
+template class BKZAutoAbort<Z_NR<mpz_t>, FP_NR<long double>>;
+
+template class BKZReduction<Z_NR<long>, FP_NR<long double>>;
+template class BKZAutoAbort<Z_NR<long>, FP_NR<long double>>;
 #endif
 
 #ifdef FPLLL_WITH_DPE
-template class BKZReduction<FP_NR<dpe_t>>;
-template class BKZAutoAbort<FP_NR<dpe_t>>;
+template class BKZReduction<Z_NR<mpz_t>, FP_NR<dpe_t>>;
+template class BKZAutoAbort<Z_NR<mpz_t>, FP_NR<dpe_t>>;
+
+template class BKZReduction<Z_NR<long>, FP_NR<dpe_t>>;
+template class BKZAutoAbort<Z_NR<long>, FP_NR<dpe_t>>;
 #endif
 
 #ifdef FPLLL_WITH_QD
-template class BKZReduction<FP_NR<dd_real>>;
-template class BKZAutoAbort<FP_NR<dd_real>>;
+template class BKZReduction<Z_NR<mpz_t>, FP_NR<dd_real>>;
+template class BKZAutoAbort<Z_NR<mpz_t>, FP_NR<dd_real>>;
 
-template class BKZReduction<FP_NR<qd_real>>;
-template class BKZAutoAbort<FP_NR<qd_real>>;
+template class BKZReduction<Z_NR<mpz_t>, FP_NR<qd_real>>;
+template class BKZAutoAbort<Z_NR<mpz_t>, FP_NR<qd_real>>;
+
+template class BKZReduction<Z_NR<long>, FP_NR<dd_real>>;
+template class BKZAutoAbort<Z_NR<long>, FP_NR<dd_real>>;
+
+template class BKZReduction<Z_NR<long>, FP_NR<qd_real>>;
+template class BKZAutoAbort<Z_NR<long>, FP_NR<qd_real>>;
 #endif
 
-template class BKZReduction<FP_NR<mpfr_t>>;
-template class BKZAutoAbort<FP_NR<mpfr_t>>;
+template class BKZReduction<Z_NR<mpz_t>, FP_NR<mpfr_t>>;
+template class BKZAutoAbort<Z_NR<mpz_t>, FP_NR<mpfr_t>>;
+
+template class BKZReduction<Z_NR<long>, FP_NR<mpfr_t>>;
+template class BKZAutoAbort<Z_NR<long>, FP_NR<mpfr_t>>;
 
 FPLLL_END_NAMESPACE
