@@ -1,4 +1,5 @@
 #include <../fplll/sieve/sieve_main.h> /* standalone bin */
+#include <../tests/test_utils.h>
 #include <cstring>
 #include <fplll.h>
 
@@ -9,34 +10,6 @@
 using namespace std;
 using namespace fplll;
 
-/**
-   @brief Read matrix from `input_filename`.
-
-   @param A
-   @param input_filename
-   @return
-*/
-template <class ZT> void read_matrix(ZZ_mat<ZT> &A, const char *input_filename)
-{
-  istream *is = new ifstream(input_filename);
-  *is >> A;
-  delete is;
-}
-
-/**
-   @brief Read vector from `input_filename` into `b`.
-
-   @param b                vector
-   @param input_filename   filename
-   @return
-*/
-template <class ZT> void read_vector(vector<Z_NR<ZT>> &b, const char *input_filename)
-{
-  istream *is = new ifstream(input_filename);
-  *is >> b;
-  delete is;
-}
-
 /**
    @brief Test sieve by checking if function returns correct vector.
 
@@ -44,18 +17,12 @@ template <class ZT> void read_vector(vector<Z_NR<ZT>> &b, const char *input_file
    @param b              shortest vector
    @return
 */
-template <class ZT> int test_sieve_alg(ZZ_mat<ZT> &A, IntVect &b, int alg)
+template <class ZT> int test_sieve_alg(ZZ_mat<ZT> &A, vector<Z_NR<mpz_t>> &b, int alg)
 {
   GaussSieve<ZT, FP_NR<double>> gsieve(A, alg, 0, 0);
   Z_NR<ZT> goal_norm;
   goal_norm = 0;
-  gsieve.set_goal_norm2(goal_norm);
-  if (gsieve.alg == 3)
-    gsieve.run_3sieve();
-  else if (gsieve.alg == 4)
-    gsieve.run_4sieve();
-  else
-    gsieve.run_2sieve();
+  gsieve.sieve(goal_norm);
   NumVect<Z_NR<ZT>> v = gsieve.return_first();
   Z_NR<ZT> tmp;
   Z_NR<ZT> norm_s;
@@ -72,7 +39,7 @@ template <class ZT> int test_sieve_alg(ZZ_mat<ZT> &A, IntVect &b, int alg)
   return 0;
 }
 
-template <class ZT> int test_sieve(ZZ_mat<ZT> &A, IntVect &b)
+template <class ZT> int test_sieve(ZZ_mat<ZT> &A, vector<Z_NR<mpz_t>> &b)
 {
   int r = 0;
   r |= test_sieve_alg<ZT>(A, b, 2);
@@ -90,10 +57,12 @@ template <class ZT> int test_sieve(ZZ_mat<ZT> &A, IntVect &b)
 template <class ZT> int test_filename(const char *input_filename, const char *output_filename)
 {
   ZZ_mat<ZT> A;
-  read_matrix(A, input_filename);
-  IntVect b;
-  read_vector(b, output_filename);
-  return test_sieve<ZT>(A, b);
+  int status = 0;
+  status |= read_matrix(A, input_filename);
+  vector<Z_NR<mpz_t>> b;
+  status |= read_vector(b, output_filename);
+  status |= test_sieve<ZT>(A, b);
+  return status;
 }
 
 /*
