@@ -14,6 +14,7 @@
    You should have received a copy of the GNU Lesser General Public License
    along with fplll. If not, see <http://www.gnu.org/licenses/>. */
 
+#include <../tests/test_utils.h>
 #include <cstring>
 #include <fplll.h>
 
@@ -31,36 +32,6 @@ enum Test
   DSVP_REDUCE
 };
 
-/**
-   @brief Read matrix from `input_filename`.
-
-   @param A
-   @param input_filename
-   @return
-*/
-
-template <class ZT> void read_matrix(ZZ_mat<ZT> &A, const char *input_filename)
-{
-  istream *is = new ifstream(input_filename);
-  *is >> A;
-  delete is;
-}
-
-/**
-   @brief Read vector from `input_filename` into `b`.
-
-   @param b                vector
-   @param input_filename   filename
-   @return
-*/
-
-template <class ZT> void read_vector(vector<Z_NR<ZT>> &b, const char *input_filename)
-{
-  istream *is = new ifstream(input_filename);
-  *is >> b;
-  delete is;
-}
-
 /**
    @brief Test if SVP function returns vector with right norm.
 
@@ -69,12 +40,12 @@ template <class ZT> void read_vector(vector<Z_NR<ZT>> &b, const char *input_file
    @return
 */
 
-template <class ZT> int test_svp(ZZ_mat<ZT> &A, IntVect &b)
+template <class ZT> int test_svp(ZZ_mat<ZT> &A, vector<Z_NR<mpz_t>> &b)
 {
-  IntVect sol_coord;   // In the LLL-reduced basis
-  IntVect sol_coord2;  // In the initial basis
-  IntVect solution;
-  IntMatrix u;
+  vector<Z_NR<mpz_t>> sol_coord;   // In the LLL-reduced basis
+  vector<Z_NR<mpz_t>> sol_coord2;  // In the initial basis
+  vector<Z_NR<mpz_t>> solution;
+  ZZ_mat<mpz_t> u;
 
   int status =
       lll_reduction(A, u, LLL_DEF_DELTA, LLL_DEF_ETA, LM_WRAPPER, FT_DEFAULT, 0, LLL_DEFAULT);
@@ -120,7 +91,8 @@ template <class ZT> int test_svp(ZZ_mat<ZT> &A, IntVect &b)
    @param b              coefficients of shortest dual vector
    @return
 */
-template <class ZT> int dual_length(Float &norm, ZZ_mat<ZT> &A, const IntVect &coords)
+template <class ZT>
+int dual_length(FP_NR<mpfr_t> &norm, ZZ_mat<ZT> &A, const vector<Z_NR<mpz_t>> &coords)
 {
   int d = coords.size();
   if (A.get_rows() != d)
@@ -129,25 +101,25 @@ template <class ZT> int dual_length(Float &norm, ZZ_mat<ZT> &A, const IntVect &c
     cerr << A.get_rows() << " vs " << d << endl;
     return 1;
   }
-  FloatVect coords_d(d);
+  vector<FP_NR<mpfr_t>> coords_d(d);
   for (int i = 0; i < d; i++)
   {
     coords_d[i] = coords[i].get_d();
   }
 
-  IntMatrix empty_mat;
-  MatGSO<Integer, Float> gso(A, empty_mat, empty_mat, GSO_INT_GRAM);
+  ZZ_mat<mpz_t> empty_mat;
+  MatGSO<Z_NR<mpz_t>, FP_NR<mpfr_t>> gso(A, empty_mat, empty_mat, GSO_INT_GRAM);
   if (!gso.update_gso())
   {
     cerr << "GSO Failure." << endl;
     return 1;
   }
-  Float tmp;
+  FP_NR<mpfr_t> tmp;
   gso.get_r(tmp, d - 1, d - 1);
   tmp.pow_si(tmp, -1);
 
-  FloatVect alpha(d);
-  Float mu, alpha2, r_inv;
+  vector<FP_NR<mpfr_t>> alpha(d);
+  FP_NR<mpfr_t> mu, alpha2, r_inv;
   norm = 0.0;
   for (int i = 0; i < d; i++)
   {
@@ -174,13 +146,13 @@ template <class ZT> int dual_length(Float &norm, ZZ_mat<ZT> &A, const IntVect &c
    @return
 */
 
-template <class ZT> int test_dual_svp(ZZ_mat<ZT> &A, IntVect &b)
+template <class ZT> int test_dual_svp(ZZ_mat<ZT> &A, vector<Z_NR<mpz_t>> &b)
 {
-  IntVect sol_coord;  // In the LLL-reduced basis
-  IntVect solution;
-  IntMatrix u;
+  vector<Z_NR<mpz_t>> sol_coord;  // In the LLL-reduced basis
+  vector<Z_NR<mpz_t>> solution;
+  ZZ_mat<mpz_t> u;
 
-  Float normb;
+  FP_NR<mpfr_t> normb;
   if (dual_length(normb, A, b))
   {
     return 1;
@@ -202,13 +174,13 @@ template <class ZT> int test_dual_svp(ZZ_mat<ZT> &A, IntVect &b)
     return status;
   }
 
-  Float norm_sol;
+  FP_NR<mpfr_t> norm_sol;
   if (dual_length(norm_sol, A, sol_coord))
   {
     return 1;
   }
 
-  Float error;
+  FP_NR<mpfr_t> error;
   error = 1;
   error.mul_2si(error, -(int)error.get_prec());
   normb += error;
@@ -228,12 +200,12 @@ template <class ZT> int test_dual_svp(ZZ_mat<ZT> &A, IntVect &b)
    @param b              shortest dual vector
    @return
 */
-template <class ZT> int test_dsvp_reduce(ZZ_mat<ZT> &A, IntVect &b)
+template <class ZT> int test_dsvp_reduce(ZZ_mat<ZT> &A, vector<Z_NR<mpz_t>> &b)
 {
-  IntMatrix u;
+  ZZ_mat<mpz_t> u;
   int d = A.get_rows();
 
-  Float normb;
+  FP_NR<mpfr_t> normb;
   if (dual_length(normb, A, b))
   {
     return 1;
@@ -247,34 +219,27 @@ template <class ZT> int test_dsvp_reduce(ZZ_mat<ZT> &A, IntVect &b)
     return status;
   }
 
-  IntMatrix empty_mat;
-  MatGSO<Integer, Float> gso(A, empty_mat, empty_mat, GSO_INT_GRAM);
-  LLLReduction<Integer, Float> lll_obj(gso, LLL_DEF_DELTA, LLL_DEF_ETA, LLL_DEFAULT);
+  ZZ_mat<mpz_t> empty_mat;
+  MatGSO<Z_NR<mpz_t>, FP_NR<mpfr_t>> gso(A, empty_mat, empty_mat, GSO_INT_GRAM);
+  LLLReduction<Z_NR<mpz_t>, FP_NR<mpfr_t>> lll_obj(gso, LLL_DEF_DELTA, LLL_DEF_ETA, LLL_DEFAULT);
 
   vector<Strategy> strategies;
   BKZParam dummy(d, strategies);
-  BKZReduction<Float> bkz_obj(gso, lll_obj, dummy);
-  bool clean = true;
+  BKZReduction<Z_NR<mpz_t>, FP_NR<mpfr_t>> bkz_obj(gso, lll_obj, dummy);
 
-  bkz_obj.svp_reduction_ex(0, d, dummy, clean, true);
-  status = bkz_obj.status;
-  if (status != RED_SUCCESS)
-  {
-    cerr << "Failure: " << get_red_status_str(status) << endl;
-    return status;
-  }
+  bkz_obj.svp_reduction(0, d, dummy, true);
 
-  Float norm_sol;
-  Integer zero;
+  FP_NR<mpfr_t> norm_sol;
+  Z_NR<mpz_t> zero;
   zero = 0;
-  IntVect e_n(d, zero);
+  vector<Z_NR<mpz_t>> e_n(d, zero);
   e_n[d - 1] = 1;
   if (dual_length(norm_sol, A, e_n))
   {
     return 1;
   }
 
-  Float error;
+  FP_NR<mpfr_t> error;
   error = 1;
   error.mul_2si(error, -(int)error.get_prec());
   normb += error;
@@ -300,19 +265,23 @@ int test_filename(const char *input_filename, const char *output_filename,
                   const Test test = SVP_ENUM)
 {
   ZZ_mat<ZT> A;
-  read_matrix(A, input_filename);
+  int status = 0;
+  status |= read_matrix(A, input_filename);
 
-  IntVect b;
-  read_vector(b, output_filename);
+  vector<Z_NR<mpz_t>> b;
+  status |= read_vector(b, output_filename);
 
   switch (test)
   {
   case SVP_ENUM:
-    return test_svp<ZT>(A, b);
+    status |= test_svp<ZT>(A, b);
+    return status;
   case DSVP_ENUM:
-    return test_dual_svp<ZT>(A, b);
+    status |= test_dual_svp<ZT>(A, b);
+    return status;
   case DSVP_REDUCE:
-    return test_dsvp_reduce<ZT>(A, b);
+    status |= test_dsvp_reduce<ZT>(A, b);
+    return status;
   }
 
   cerr << "Unknown test." << endl;
