@@ -1,16 +1,25 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 from optparse import OptionParser
+import json
 
 def do_plot(filename):
     import matplotlib.pyplot as plt
     x1,x2,y1,y2 = None,None,None,None
-    for i,line in enumerate(open(filename).readlines()):
+    i = 0
+    filer = open(filename, "r")
+    data_json = json.load(filer)
+    filer.close()
+
+    for d in data_json:
 
         plt.clf()
-        line = line.strip()
-        name, data = line.split(":")
-        data = [d for d in data.split(" ") if d]
+        name = d.get("step")
+        if int(d.get("loop")) != -1:
+            name += " {0}".format(d.get("loop"))
+        if d.get("step") != "Input":
+            name += " ({0} s)".format(d.get("time"))
+        data = d.get("norms")
         data = map(float, data)
         
         plt.plot(range(len(data)), data)
@@ -25,6 +34,7 @@ def do_plot(filename):
         from os.path import basename
         name = basename(filename)
         plt.savefig("%s-%04d.png"%(name,i))
+        i += 1
 
 def main():
     parser = OptionParser(usage="""\
@@ -40,4 +50,4 @@ def main():
     do_plot(filename)
 
 if __name__ == '__main__':
-    main()
\ No newline at end of file
+    main()
